Package: hdf5r
Type: Package
Title: Interface to the 'HDF5' Binary Data Format
Version: 1.3.12
Authors@R: c(
    person("Holger", "Hoefling", email = "hhoeflin@gmail.com", role = c("aut", "cre")),
    person("Mario", "Annau", email = "mario.annau@gmail.com", role = "aut"),
    person("Novartis Institute for BioMedical Research (NIBR)", role = "cph")
    )
Description: 'HDF5' is a data model, library and file format for storing
    and managing large amounts of data. This package provides a nearly
    feature complete, object oriented  wrapper for the 'HDF5' API
    <https://support.hdfgroup.org/documentation/hdf5/latest/_r_m.html> using R6 classes.
    Additionally, functionality is added so that 'HDF5' objects behave very
    similar to their corresponding R counterparts.
URL: https://hhoeflin.github.io/hdf5r/,
        https://github.com/hhoeflin/hdf5r/
BugReports: https://github.com/hhoeflin/hdf5r/issues
License: Apache License 2.0 | file LICENSE
Copyright: For the hdf5r package: Novartis Institute for BioMedical
        Research Inc. For HDF5: see the HDF5_COPYRIGHTS file.
LazyLoad: true
Depends: R (>= 3.2.2), methods
Imports: R6, bit64, utils
Suggests: testthat, knitr, rmarkdown, nycflights13, reshape2, formatR,
SystemRequirements: HDF5 (>= 1.8.13)
VignetteBuilder: knitr
NeedsCompilation: yes
RoxygenNote: 6.1.1.9000
Collate: 'Common_functions.R' 'Compound.R' 'H5constants.R'
        'Helper_functions.R' 'Misc.R' 'R6Classes.R' 'R6Classes_H5A.R'
        'R6Classes_H5D.R' 'R6Classes_H5File.R' 'R6Classes_H5Group.R'
        'R6Classes_H5P.R' 'R6Classes_H5R.R' 'R6Classes_H5S.R'
        'R6Classes_H5T.R' 'adapt_during_onLoad.R' 'convert.R'
        'factor_ext.R' 'globalVariables.R' 'h5errorHandling.R'
        'h5wrapper.R' 'hdf5r.R' 'high_level_UI.R' 'open_objs.R' 'zzz.R'
Packaged: 2025-01-19 16:02:58 UTC; hhoeflin
Author: Holger Hoefling [aut, cre],
  Mario Annau [aut],
  Novartis Institute for BioMedical Research (NIBR) [cph]
Maintainer: Holger Hoefling <hhoeflin@gmail.com>
Repository: CRAN
Date/Publication: 2025-01-20 11:00:05 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 00:17:21 UTC; unix
Archs: hdf5r.so.dSYM
