# Generated by roxygen2: do not edit by hand

S3method("!=",factor_ext)
S3method("$",types_env)
S3method("==",factor_ext)
S3method("[",H5D)
S3method("[",H5R)
S3method("[",H5S)
S3method("[",factor_ext)
S3method("[<-",H5D)
S3method("[<-",H5R)
S3method("[<-",factor_ext)
S3method("[[",H5File)
S3method("[[",H5Group)
S3method("[[",factor_ext)
S3method("[[",types_env)
S3method("[[<-",H5File)
S3method("[[<-",H5Group)
S3method("[[<-",factor_ext)
S3method("dim<-",H5R)
S3method("dimnames<-",H5R)
S3method(as.character,factor_ext)
S3method(as.data.frame,H5R)
S3method(as.vector,H5R)
S3method(c,H5D)
S3method(c,H5R)
S3method(c,factor_ext)
S3method(cbind,H5D)
S3method(cbind,H5R)
S3method(dim,H5R)
S3method(dimnames,H5R)
S3method(format,H5R)
S3method(length,H5R)
S3method(names,H5File)
S3method(names,H5Group)
S3method(names,H5R)
S3method(print,H5R)
S3method(print,data.frame_ext)
S3method(print,factor_ext)
S3method(rbind,H5D)
S3method(rbind,H5R)
S3method(t,H5R)
S3method(values,default)
S3method(values,factor)
S3method(values,factor_ext)
export("h5attr<-")
export(H5A)
export(H5D)
export(H5File)
export(H5Group)
export(H5P)
export(H5P_ATTRIBUTE_CREATE)
export(H5P_CLASS)
export(H5P_DATASET_ACCESS)
export(H5P_DATASET_CREATE)
export(H5P_DATASET_XFER)
export(H5P_FILE_ACCESS)
export(H5P_FILE_CREATE)
export(H5P_LINK_ACCESS)
export(H5P_LINK_CREATE)
export(H5P_OBJECT_COPY)
export(H5P_OBJECT_CREATE)
export(H5R)
export(H5R_DATASET_REGION)
export(H5R_OBJECT)
export(H5RefClass)
export(H5S)
export(H5T)
export(H5T_ARRAY)
export(H5T_COMPLEX)
export(H5T_COMPOUND)
export(H5T_ENUM)
export(H5T_FLOAT)
export(H5T_INTEGER)
export(H5T_LOGICAL)
export(H5T_STRING)
export(H5T_VLEN)
export(as_hex)
export(coerce_to_factor)
export(coercible_to_factor)
export(createDataSet)
export(createGroup)
export(create_empty)
export(existsGroup)
export(extendDataSet)
export(factor_ext)
export(flatten_df)
export(guess_chunks)
export(guess_dim)
export(guess_dtype)
export(guess_nelem)
export(guess_space)
export(h5attr)
export(h5attr_names)
export(h5attributes)
export(h5close)
export(h5const)
export(h5file)
export(h5flush)
export(h5garbage_collect)
export(h5types)
export(h5unlink)
export(h5version)
export(is.H5R)
export(is.H5R_DATASET_REGION)
export(is.H5R_OBJECT)
export(is.factor_ext)
export(is.h5file)
export(is_hdf5)
export(list.attributes)
export(list.datasets)
export(list.groups)
export(list.objects)
export(openGroup)
export(openLocation)
export(readDataSet)
export(subset_assign_h5.H5D)
export(subset_h5.H5D)
export(subset_h5.H5S)
export(text_to_dtype)
export(values)
importFrom(R6,R6Class)
importFrom(bit64,as.integer64)
importFrom(bit64,integer64)
importFrom(bit64,is.integer64)
importFrom(bit64,is.na.integer64)
importFrom(methods,setOldClass)
importFrom(utils,compareVersion)
importFrom(utils,packageVersion)
useDynLib(hdf5r, .registration = TRUE)
