# On Release

On release, run tests on the following sites:

## Check reverse dependencies

Any reverse dependencies should be checked before merging.

## For linux

Run it in my local docker.

Also use the new R-hub with Github actions.


## For Mac

[Mac builder](https://mac.r-project.org/macbuilder/submit.html)


## For windows

[Win builder](https://win-builder.r-project.org/)
